# Copyright (c) 2013, 2014 Erik Hetzner
#
# This file is part of ledger-autosync
#
# ledger-autosync is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# ledger-autosync is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ledger-autosync. If not, see
# <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from ledgerautosync.converter import OfxConverter
from ledgerautosync.ledgerwrap import Ledger
import os.path
from decimal import Decimal

from ofxparse import OfxParser

from nose.plugins.attrib import attr
from tests import LedgerTestCase


@attr('generic')
class TestOfxConverter(LedgerTestCase):
    def test_checking(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'checking.ofx')))
        converter = OfxConverter(ofx=ofx, name="Foo")
        self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[0]).format(),
"""2011/03/31 DIVIDEND EARNED FOR PERIOD OF 03/01/2011 THROUGH 03/31/2011 ANNUAL PERCENTAGE YIELD EARNED IS 0.05%
  ; ofxid: 1101.1452687~7.0000486
  Foo  $0.01
  Expenses:Misc  -$0.01
""")
        self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[1]).format(),
"""2011/04/05 AUTOMATIC WITHDRAWAL, ELECTRIC BILL WEB(S )
  ; ofxid: 1101.1452687~7.0000487
  Foo  -$34.51
  Expenses:Misc  $34.51
""")

        self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[2]).format(),
"""2011/04/07 RETURNED CHECK FEE, CHECK # 319 FOR $45.33 ON 04/07/11
  ; ofxid: 1101.1452687~7.0000488
  Foo  -$25.00
  Expenses:Misc  $25.00
""")

    def test_indent(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'checking.ofx')))
        converter = OfxConverter(ofx=ofx, name="Foo", indent=4)
        # testing indent, so do not use the string collapsing version of assert
        self.assertEqual(converter.convert(ofx.account.statement.transactions[0]).format(),
"""2011/03/31 DIVIDEND EARNED FOR PERIOD OF 03/01/2011 THROUGH 03/31/2011 ANNUAL PERCENTAGE YIELD EARNED IS 0.05%
    ; ofxid: 1101.1452687~7.0000486
    Foo                                                     $0.01
    Expenses:Misc                                          -$0.01
""")

    def test_investments(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'fidelity.ofx')))
        converter = OfxConverter(ofx=ofx, name="Foo")
        self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[0]).format(),
"""2012/07/20 YOU BOUGHT
  ; ofxid: 7776.01234567890.0123456789020201120120720
  Foo  100.00000 INTC @ $25.635000000
  Assets:Unknown  -$2563.50
""")
        # test no payee/memo
        self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[1]).format(),
"""2012/07/27 Foo: buystock
  ; ofxid: 7776.01234567890.0123456789020901120120727
  Foo  128.00000 SDRL @ $39.390900000
  Assets:Unknown  -$5042.04
""")

    def test_dynamic_account(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'checking.ofx')))
        ledger = Ledger(os.path.join('fixtures', 'checking-dynamic-account.lgr'))
        converter = OfxConverter(ofx=ofx, name="Assets:Foo", ledger=ledger)
        self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[1]).format(),
"""2011/04/05 AUTOMATIC WITHDRAWAL, ELECTRIC BILL WEB(S )
  ; ofxid: 1101.1452687~7.0000487
  Assets:Foo  -$34.51
  Expenses:Bar  $34.51
""")

    def test_balance_assertion(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'checking.ofx')))
        ledger = Ledger(os.path.join('fixtures', 'checking.lgr'))
        converter = OfxConverter(ofx=ofx, name="Assets:Foo", ledger=ledger)
        self.assertEqualLedgerPosting(converter.format_balance(ofx.account.statement),
"""2013/05/25 * --Autosync Balance Assertion
  Assets:Foo  $0.00 = $100.99
""")

    def test_initial_balance(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'checking.ofx')))
        ledger = Ledger(os.path.join('fixtures', 'checking.lgr'))
        converter = OfxConverter(ofx=ofx, name="Assets:Foo", ledger=ledger)
        self.assertEqualLedgerPosting(converter.format_initial_balance(ofx.account.statement),
"""2000/01/01 * --Autosync Initial Balance
  ; ofxid: 1101.1452687~7.autosync_initial
  Assets:Foo  $160.49
  Assets:Equity  -$160.49
""")

    def test_unknownaccount(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'checking.ofx')))
        converter = OfxConverter(ofx=ofx, name="Foo",
                                 unknownaccount='Expenses:Unknown')
        self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[0]).format(),
"""2011/03/31 DIVIDEND EARNED FOR PERIOD OF 03/01/2011 THROUGH 03/31/2011 ANNUAL PERCENTAGE YIELD EARNED IS 0.05%
  ; ofxid: 1101.1452687~7.0000486
  Foo  $0.01
  Expenses:Unknown  -$0.01
""")

    def test_quote_commodity(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'fidelity.ofx')))
        converter = OfxConverter(ofx=ofx, name="Foo")
        self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[0]).format(),
"""2012/07/20 YOU BOUGHT
  ; ofxid: 7776.01234567890.0123456789020201120120720
  Foo  100.00000 INTC @ $25.635000000
  Assets:Unknown  -$2563.50
""")

    # Check that <TRANSFER> txns are parsed.
    def test_transfer_txn(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'investment_401k.ofx')))
        converter = OfxConverter(ofx=ofx, name="Foo",
                                 unknownaccount='Expenses:Unknown')
        if len(ofx.account.statement.transactions) > 2:
            # older versions of ofxparse would skip these transactions
            if hasattr(ofx.account.statement.transactions[2], 'tferaction'):
                # unmerged pull request
                self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[2]).format(),
"""2014/06/30 Foo: transfer: out
    ; ofxid: 1234.12345678.123456-01.3
    Foo  -9.060702 BAZ @ $21.928764
    Transfer  $198.69
""")
            else:
                            self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[2]).format(),
"""2014/06/30 Foo: transfer
    ; ofxid: 1234.12345678.123456-01.3
    Foo  -9.060702 BAZ @ $21.928764
    Transfer  $198.69
""")


    def test_position(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'cusip.ofx')))
        converter = OfxConverter(ofx=ofx, name="Foo", indent=4,
                                 unknownaccount='Expenses:Unknown')
        self.assertEqual(converter.format_position(ofx.account.statement.positions[0]),
                         """P 2016/10/08 07:30:08 SHSAX 47.8600000
""")

    def test_dividend(self):
        ofx = OfxParser.parse(file(os.path.join('fixtures', 'income.ofx')))
        converter = OfxConverter(ofx=ofx, name="Foo")
        self.assertEqualLedgerPosting(converter.convert(ofx.account.statement.transactions[0]).format(),
"""2016/10/12 DIVIDEND RECEIVED
    ; dividend_from: cusip_redacted
    ; ofxid: 1234.12345678.123456-01.redacted
    Foo                                     $1234.56
    Income:Dividends                       -$1234.56
""")



# Copyright (c) 2013, 2014 Erik Hetzner
#
# This file is part of ledger-autosync
#
# ledger-autosync is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# ledger-autosync is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ledger-autosync. If not, see
# <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from ledgerautosync.cli import run
from ledgerautosync.converter import OfxConverter
from ledgerautosync.ledgerwrap import Ledger, HLedger, LedgerPython
from ledgerautosync.sync import OfxSynchronizer
from ledgerautosync import EmptyInstitutionException
import os.path
from ofxclient.config import OfxConfig

from unittest import TestCase
from nose.plugins.attrib import attr
from nose.tools import raises


class WeirdOfxTest(object):
    @raises(EmptyInstitutionException)
    def test_no_institution_no_fid(self):
        config = OfxConfig(os.path.join('fixtures', 'ofxclient.ini'))
        run([os.path.join('fixtures', 'no-institution.ofx'),
             '-l', os.path.join('fixtures', 'empty.lgr'),
             '-a', 'Assets:Savings:Foo'], config)

    def test_no_institution(self):
        ofxpath = os.path.join('fixtures', 'no-institution.ofx')
        OfxSynchronizer(self.lgr).parse_file(ofxpath)

    @raises(EmptyInstitutionException)
    def test_no_institution_no_accountname(self):
        ofxpath = os.path.join('fixtures', 'no-institution.ofx')
        (ofx, txns) = OfxSynchronizer(self.lgr).parse_file(ofxpath)
        OfxConverter(ofx, name=None)

    def test_apostrophe(self):
        ofxpath = os.path.join('fixtures', 'apostrophe.ofx')
        OfxSynchronizer(self.lgr).parse_file(ofxpath)

    def test_one_settleDate(self):
        ofxpath = os.path.join('fixtures', 'fidelity-one-dtsettle.ofx')
        OfxSynchronizer(self.lgr).parse_file(ofxpath)


@attr('hledger')
class TestWeirdOfxHledger(TestCase, WeirdOfxTest):
    def setUp(self):
        self.lgr = HLedger(os.path.join('fixtures', 'empty.lgr'))


@attr('ledger')
class TestWeirdOfxLedger(TestCase, WeirdOfxTest):
    def setUp(self):
        self.lgr = Ledger(os.path.join('fixtures', 'empty.lgr'), no_pipe=True)


@attr('ledger-python')
class TestWeirdOfxLedgerPython(TestCase, WeirdOfxTest):
    def setUp(self):
        self.lgr = LedgerPython(os.path.join('fixtures', 'empty.lgr'))



# Copyright (c) 2013-2016 Erik Hetzner
#
# This file is part of ledger-autosync
#
# ledger-autosync is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# ledger-autosync is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ledger-autosync. If not, see
# <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from ledgerautosync.converter import Converter, CsvConverter, AmazonConverter, MintConverter, PaypalConverter, Amount, Posting
from decimal import Decimal
import hashlib
import csv

from nose.plugins.attrib import attr
from tests import LedgerTestCase


@attr('generic')
class TestPosting(LedgerTestCase):
    def test_format(self):
        self.assertRegexpMatches(
            Posting(
                "Foo",
                Amount(Decimal("10.00"), "$")
            ).format(indent=2),
            r'^  Foo.*$')


@attr('generic')
class TestAmount(LedgerTestCase):
    def test_amount(self):
        self.assertEqual(
            "$10.00",
            Amount(Decimal("10.001"), "$").format(),
            "Formats to 2 points precision by default")
        self.assertEqual(
            "10.00 USD",
            Amount(Decimal(10), "USD").format(),
            "Longer commodity names come after")
        self.assertEqual(
            "-$10.00",
            Amount(Decimal(10), "$", reverse=True).format(),
            "Reverse flag works.")
        self.assertEqual(
            "10.00 \"ABC123\"",
            Amount(Decimal(10), "ABC123").format(),
            "Currencies with numbers are quoted")
        self.assertEqual(
            "10.00 \"A BC\"",
            Amount(Decimal(10), "A BC").format(),
            "Currencies with whitespace are quoted")
        self.assertEqual(
            "$10.001",
            Amount(Decimal("10.001"), "$", unlimited=True).format())


@attr('generic')
class TestCsvConverter(LedgerTestCase):
    def test_get_csv_id(self):
        converter = CsvConverter(None)
        h = {'foo': 'bar', 'bar': 'foo'}
        self.assertEqual(converter.get_csv_id(h),
                         hashlib.md5("bar=foo\nfoo=bar\n").hexdigest())


@attr('generic')
class TestPaypalConverter(LedgerTestCase):
    def test_format(self):
        with open('fixtures/paypal.csv', 'rb') as f:
            dialect = csv.Sniffer().sniff(f.read(1024))
            f.seek(0)
            dialect.skipinitialspace = True
            reader = csv.DictReader(f, dialect=dialect)
            converter = CsvConverter.make_converter(reader, name='Foo')
            self.assertEqual(type(converter), PaypalConverter)
            self.assertEqual(
                converter.convert(reader.next()).format(),
                """2016/06/04 Jane Doe someone@example.net My Friend ID: XYZ1, Recurring Payment Sent
    ; csvid: paypal.XYZ1
    Foo                                                -20.00 USD
    Expenses:Misc                                       20.00 USD
""")
            self.assertEqual(
                converter.convert(reader.next()).format(),
                """2016/06/04 Debit Card ID: XYZ2, Charge From Debit Card
    ; csvid: paypal.XYZ2
    Foo                                                 20.00 USD
    Transfer:Paypal                                    -20.00 USD
""")

@attr('generic')
class TestAmazonConverter(LedgerTestCase):
    def test_format(self):
        with open('fixtures/amazon.csv', 'rb') as f:
            dialect = csv.Sniffer().sniff(f.read(1024))
            f.seek(0)
            dialect.skipinitialspace = True
            reader = csv.DictReader(f, dialect=dialect)
            converter = CsvConverter.make_converter(reader, name='Foo')
            self.assertEqual(type(converter), AmazonConverter)
            self.assertEqual(
                converter.convert(reader.next()).format(),
                """2016/01/29 Best Soap Ever
    ; url: https://www.amazon.com/gp/css/summary/print.html/ref=od_aui_print_invoice?ie=UTF8&orderID=123-4567890-1234567
    ; csvid: amazon.123-4567890-1234567
    Foo                                                    $21.90
    Expenses:Misc                                         -$21.90
""")

@attr('generic')
class TestMintConverter(LedgerTestCase):
    def test_format(self):
        with open('fixtures/mint.csv', 'rb') as f:
            dialect = csv.Sniffer().sniff(f.read(1024))
            f.seek(0)
            dialect.skipinitialspace = True
            reader = csv.DictReader(f, dialect=dialect)
            converter = CsvConverter.make_converter(reader)
            self.assertEqual(type(converter), MintConverter)
            self.assertEqual(
                converter.convert(reader.next()).format(),
                """2016/08/02 Amazon
    ; csvid: mint.a7c028a73d76956453dab634e8e5bdc1
    1234                                                   $29.99
    Expenses:Shopping                                     -$29.99
""")
            self.assertEqual(
                converter.convert(reader.next()).format(),
                """2016/06/02 Autopay Rautopay Auto
    ; csvid: mint.a404e70594502dd62bfc6f15d80b7cd7
    1234                                                 -$123.45
    Credit Card Payment                                   $123.45
""")



#!/usr/bin/env python

# Copyright (c) 2013, 2014 Erik Hetzner
# Portions Copyright (c) 2016 James S Blachly, MD
#
# This file is part of ledger-autosync
#
# ledger-autosync is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# ledger-autosync is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ledger-autosync. If not, see
# <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from ofxclient.config import OfxConfig
import argparse
import csv
from ledgerautosync import EmptyInstitutionException, LedgerAutosyncException
from ledgerautosync.converter import OfxConverter, CsvConverter, AUTOSYNC_INITIAL, \
    ALL_AUTOSYNC_INITIAL
from ledgerautosync.converter import SecurityList
from ledgerautosync.sync import OfxSynchronizer, CsvSynchronizer
from ledgerautosync.ledgerwrap import mk_ledger, Ledger, HLedger, LedgerPython
import logging
import re
import sys
import traceback
import os
import os.path
import imp


def find_ledger_file(ledgerrcpath=None):
    """Returns main ledger file path or raise exception if it cannot be \
found."""
    if ledgerrcpath is None:
        ledgerrcpath = os.path.abspath(os.path.expanduser("~/.ledgerrc"))
    if "LEDGER_FILE" in os.environ:
        return os.path.abspath(os.path.expanduser(os.environ["LEDGER_FILE"]))
    elif os.path.exists(ledgerrcpath):
        # hacky
        ledgerrc = open(ledgerrcpath)
        for line in ledgerrc.readlines():
            md = re.match(r"--file\s+([^\s]+).*", line)
            if md is not None:
                return os.path.abspath(os.path.expanduser(md.group(1)))
    else:
        return None


def print_results(converter, ofx, ledger, txns, args):
    """
    This function is the final common pathway of program:

    Print initial balance if requested;
    Print transactions surviving de-duplication filter;
    Print balance assertions if requested;
    Print commodity prices obtained from position statements
    """

    if args.initial:
        if (not(ledger.check_transaction_by_id
                ("ofxid", converter.mk_ofxid(AUTOSYNC_INITIAL))) and
                not(ledger.check_transaction_by_id("ofxid", ALL_AUTOSYNC_INITIAL))):
            print converter.format_initial_balance(ofx.account.statement)
    for txn in txns:
        print converter.convert(txn).format(args.indent)
    if args.assertions:
        print converter.format_balance(ofx.account.statement)

    # if OFX has positions use these to obtain commodity prices
    # and print "P" records to provide dated/timed valuations
    # Note that this outputs only the commodity price,
    # not your position (e.g. # shares), even though this is in the OFX record
    if hasattr(ofx.account.statement, 'positions'):
        for pos in ofx.account.statement.positions:
            print converter.format_position(pos)

def sync(ledger, accounts, args):
    sync = OfxSynchronizer(ledger)
    for acct in accounts:
        try:
            (ofx, txns) = sync.get_new_txns(acct, resync=args.resync,
                                            max_days=args.max)
            if ofx is not None:
                converter = OfxConverter(ofx=ofx,
                                         name=acct.description,
                                         ledger=ledger,
                                         indent=args.indent,
                                         unknownaccount=args.unknownaccount)
                print_results(converter, ofx, ledger, txns, args)
        except KeyboardInterrupt:
            raise
        except:
            sys.stderr.write("Caught exception processing %s" %
                             (acct.description))
            traceback.print_exc(file=sys.stderr)


def import_ofx(ledger, args):
    sync = OfxSynchronizer(ledger)
    (ofx, txns) = sync.parse_file(args.PATH)
    accountname = args.account
    if accountname is None:
        if ofx.account.institution is not None:
            accountname = "%s:%s" % (ofx.account.institution.organization,
                                     ofx.account.account_id)
        else:
            raise EmptyInstitutionException("Institution provided by OFX is \
empty and no accountname supplied!")

    converter = OfxConverter(ofx=ofx,
                             name=accountname,
                             ledger=ledger,
                             indent=args.indent,
                             fid=args.fid,
                             unknownaccount=args.unknownaccount)

    print_results(converter, ofx, ledger, txns, args)


def import_csv(ledger, args):
    if args.account is None:
        raise Exception("When importing a CSV file, you must specify an account name.")
    sync = CsvSynchronizer(ledger)
    accountname = args.account
    for txn in sync.parse_file(args.PATH, accountname=args.account):
        print txn.format(args.indent)

def load_plugins(config_dir):
    plugin_dir = os.path.join(config_dir, 'ledger-autosync', 'plugins')
    if os.path.isdir(plugin_dir):
        for plugin in filter(re.compile('.py$', re.IGNORECASE).search, os.listdir(plugin_dir)):
            # Quiet loader
            import ledgerautosync.plugins
            path = os.path.join(plugin_dir, plugin)
            imp.load_source('ledgerautosync.plugins.%s'%(os.path.splitext(plugin)[0]), path)

def run(args=None, config=None):
    if args is None:
        args = sys.argv[1:]

    parser = argparse.ArgumentParser(description='Synchronize ledger.')
    parser.add_argument('-m', '--max', type=int, default=90,
                        help='maximum number of days to process')
    parser.add_argument('-r', '--resync', action='store_true', default=False,
                        help='do not stop until max days reached')
    parser.add_argument('PATH', nargs='?', help='do not sync; import from OFX \
file')
    parser.add_argument('-a', '--account', type=str, default=None,
                        help='sync only the named account; \
if importing from file, set account name for import')
    parser.add_argument('-l', '--ledger', type=str, default=None,
                        help='specify ledger file to READ for syncing')
    parser.add_argument('-L', dest='no_ledger', action='store_true', default=False,
                        help='do not de-duplicate against a ledger file')
    parser.add_argument('-i', '--indent', type=int, default=4,
                        help='number of spaces to use for indentation')
    parser.add_argument('--initial', action='store_true', default=False,
                        help='create initial balance entries')
    parser.add_argument('--fid', type=int, default=None,
                        help='pass in fid value for OFX files that do not \
supply it')
    parser.add_argument('--unknown-account', type=str, dest='unknownaccount',
                        default=None,
                        help='specify account name to use when one can\'t be \
found by payee')
    parser.add_argument('--assertions', action='store_true', default=False,
                        help='create balance assertion entries')
    parser.add_argument('-d', '--debug', action='store_true', default=False,
                        help='enable debug logging')
    parser.add_argument('--hledger', action='store_true', default=False,
                        help='force use of hledger (on by default if invoked \
as hledger-autosync)')
    parser.add_argument('--python', action='store_true', default=False,
                        help='use the ledger python interface')
    parser.add_argument('--slow', action='store_true', default=False,
                        help='use slow, but possibly more robust, method of \
calling ledger (no subprocess)')
    parser.add_argument('--which', action='store_true', default=False,
                        help='display which version of ledger (cli), hledger, \
or ledger (python) will be used by ledger-autosync to check for previous \
transactions')
    args = parser.parse_args(args)
    if sys.argv[0][-16:] == "hledger-autosync":
        args.hledger = True

    ledger_file = None
    if args.ledger and args.no_ledger:
        raise LedgerAutosyncException('You cannot specify a ledger file and -L')
    elif args.ledger:
        ledger_file = args.ledger
    else:
        ledger_file = find_ledger_file()
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    if ledger_file is None:
        sys.stderr.write("LEDGER_FILE environment variable not set, and no \
.ledgerrc file found, and -l argument was not supplied: running with deduplication disabled. \
All transactions will be printed!")
        ledger = None
    elif args.no_ledger:
        ledger = None
    elif args.hledger:
        ledger = HLedger(ledger_file)
    elif args.python:
        ledger = LedgerPython(ledger_file=ledger_file)
    elif args.slow:
        ledger = Ledger(ledger_file=ledger_file, no_pipe=True)
    else:
        ledger = mk_ledger(ledger_file)

    if args.which:
        sys.stderr.write("ledger-autosync is using ")
        if type(ledger) == Ledger:
            sys.stderr.write("ledger (cli)\n")
        elif type(ledger) == HLedger:
            sys.stderr.write("hledger\n")
        elif type(ledger) == LedgerPython:
            sys.stderr.write("ledger.so (python)\n")
        exit()

    config_dir = os.environ.get('XDG_CONFIG_HOME',
                                os.path.join(os.path.expanduser("~"),
                                             '.config'))

    load_plugins(config_dir)

    if args.PATH is None:
        if config is None:
            config_file = os.path.join(config_dir, 'ofxclient.ini')
            if (os.path.exists(config_file)):
                config = OfxConfig(file_name=config_file)
            else:
                config = OfxConfig()
        accounts = config.accounts()
        if args.account:
            accounts = [acct for acct in accounts
                        if acct.description == args.account]
        sync(ledger, accounts, args)
    else:
        _, file_extension = os.path.splitext(args.PATH)
        if file_extension == '.csv':
            import_csv(ledger, args)
        else:
            import_ofx(ledger, args)

if __name__ == '__main__':
    run()



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ledger-autosync',
    version="0.3.5",
    description="Automatically sync your bank's data with ledger",
    long_description=long_description,
    author='Erik Hetzner',
    author_email='egh@e6h.org',
    url='https://gitlab.com/egh/ledger-autosync',
    license='GPLv3',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Office/Business :: Financial :: Accounting',
        'Topic :: Office/Business :: Financial :: Investment',
        'Topic :: Office/Business :: Financial'
    ],

    keywords='ledger accounting',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=[
        'setuptools>=26',
        'ofxclient',
        'ofxparse>=0.14',
        'BeautifulSoup4',
        'fuzzywuzzy'
    ],

    extras_require={
        'test': ['nose>=1.0', 'mock']
    },

    entry_points={
        'console_scripts': [
            'ledger-autosync = ledgerautosync.cli:run',
            'hledger-autosync = ledgerautosync.cli:run'
        ]
    },

    test_suite = 'nose.collector'
)
